<?php
/**
 * @link http://craftcms.com/
 * @copyright Copyright (c) Pixel & Tonic, Inc.
 * @license http://craftcms.com/license
 */

namespace craft\behaviors;

use yii\base\Behavior;

/**
 * Custom field behavior
 *
 * This class provides attributes for all the unique custom field handles.
 *
 * @method $this titleTag(mixed $value) Sets the [[titleTag]] property
 * @method $this metaDescription(mixed $value) Sets the [[metaDescription]] property
 * @method $this metaRobotsTag(mixed $value) Sets the [[metaRobotsTag]] property
 * @method $this contentMatrix(mixed $value) Sets the [[contentMatrix]] property
 * @method $this richText(mixed $value) Sets the [[richText]] property
 * @method $this featuredTitle(mixed $value) Sets the [[featuredTitle]] property
 * @method $this featuredText(mixed $value) Sets the [[featuredText]] property
 * @method $this featuredTitleType(mixed $value) Sets the [[featuredTitleType]] property
 * @method $this faqContentBlock(mixed $value) Sets the [[faqContentBlock]] property
 * @method $this featuredContent(mixed $value) Sets the [[featuredContent]] property
 * @method $this faqTitle(mixed $value) Sets the [[faqTitle]] property
 * @method $this headingType(mixed $value) Sets the [[headingType]] property
 * @method $this articleImage(mixed $value) Sets the [[articleImage]] property
 * @method $this featuredImage(mixed $value) Sets the [[featuredImage]] property
 */
class CustomFieldBehavior extends Behavior
{
    /**
     * @var bool Whether the behavior should provide methods based on the field handles.
     */
    public $hasMethods = false;

    /**
     * @var bool Whether properties on the class should be settable directly.
     */
    public $canSetProperties = true;

    /**
     * @var string[] List of supported field handles.
     */
    public static $fieldHandles = [
        'titleTag' => true,
        'metaDescription' => true,
        'metaRobotsTag' => true,
        'contentMatrix' => true,
        'richText' => true,
        'featuredTitle' => true,
        'featuredText' => true,
        'featuredTitleType' => true,
        'faqContentBlock' => true,
        'featuredContent' => true,
        'faqTitle' => true,
        'headingType' => true,
        'articleImage' => true,
        'featuredImage' => true,
    ];

    /**
     * @var string|null Value for field with the handle “titleTag”.
     */
    public $titleTag;

    /**
     * @var string|null Value for field with the handle “metaDescription”.
     */
    public $metaDescription;

    /**
     * @var bool Value for field with the handle “metaRobotsTag”.
     */
    public $metaRobotsTag;

    /**
     * @var \craft\elements\db\MatrixBlockQuery Value for field with the handle “contentMatrix”.
     */
    public $contentMatrix;

    /**
     * @var mixed Value for field with the handle “richText”.
     */
    public $richText;

    /**
     * @var string|null Value for field with the handle “featuredTitle”.
     */
    public $featuredTitle;

    /**
     * @var mixed Value for field with the handle “featuredText”.
     */
    public $featuredText;

    /**
     * @var \craft\fields\data\SingleOptionFieldData Value for field with the handle “featuredTitleType”.
     */
    public $featuredTitleType;

    /**
     * @var \verbb\supertable\elements\db\SuperTableBlockQuery Value for field with the handle “faqContentBlock”.
     */
    public $faqContentBlock;

    /**
     * @var mixed Value for field with the handle “featuredContent”.
     */
    public $featuredContent;

    /**
     * @var string|null Value for field with the handle “faqTitle”.
     */
    public $faqTitle;

    /**
     * @var \craft\fields\data\SingleOptionFieldData Value for field with the handle “headingType”.
     */
    public $headingType;

    /**
     * @var \craft\elements\db\AssetQuery Value for field with the handle “articleImage”.
     */
    public $articleImage;

    /**
     * @var \craft\elements\db\AssetQuery Value for field with the handle “featuredImage”.
     */
    public $featuredImage;

    /**
     * @var array Additional custom field values we don’t know about yet.
     */
    private $_customFieldValues = [];

    /**
     * @inheritdoc
     */
    public function __call($name, $params)
    {
        if ($this->hasMethods && isset(self::$fieldHandles[$name]) && count($params) === 1) {
            $this->$name = $params[0];
            return $this->owner;
        }
        return parent::__call($name, $params);
    }

    /**
     * @inheritdoc
     */
    public function hasMethod($name)
    {
        if ($this->hasMethods && isset(self::$fieldHandles[$name])) {
            return true;
        }
        return parent::hasMethod($name);
    }

    /**
     * @inheritdoc
     */
    public function __isset($name)
    {
        if (isset(self::$fieldHandles[$name])) {
            return true;
        }
        return parent::__isset($name);
    }

    /**
     * @inheritdoc
     */
    public function __get($name)
    {
        if (isset(self::$fieldHandles[$name])) {
            return $this->_customFieldValues[$name] ?? null;
        }
        return parent::__get($name);
    }

    /**
     * @inheritdoc
     */
    public function __set($name, $value)
    {
        if (isset(self::$fieldHandles[$name])) {
            $this->_customFieldValues[$name] = $value;
            return;
        }
        parent::__set($name, $value);
    }

    /**
     * @inheritdoc
     */
    public function canGetProperty($name, $checkVars = true)
    {
        if ($checkVars && isset(self::$fieldHandles[$name])) {
            return true;
        }
        return parent::canGetProperty($name, $checkVars);
    }

    /**
     * @inheritdoc
     */
    public function canSetProperty($name, $checkVars = true)
    {
        if (!$this->canSetProperties) {
            return false;
        }
        if ($checkVars && isset(self::$fieldHandles[$name])) {
            return true;
        }
        return parent::canSetProperty($name, $checkVars);
    }
}
